#include "stdafx.h"

#include "Controller.h"

#include <gl/gl.h>

// The numbers are defined in the EXT_separate_specular_color specification.

#ifndef	GL_SAMPLE_BUFFERS_ARB
	#define	GL_SAMPLE_BUFFERS_ARB			0x80A8
	#define	GL_MULTISAMPLE_ARB				0x809D
#endif

#ifndef GL_VERTEX_PROGRAM_TWO_SIDE
	#define GL_VERTEX_PROGRAM_TWO_SIDE        0x8643
#endif

#ifndef GL_VERTEX_SHADER
	#define GL_VERTEX_SHADER                  0x8B31
	#define GL_FRAGMENT_SHADER                0x8B30
#endif

#ifndef	GL_COMPILE_STATUS
	#define GL_COMPILE_STATUS                 0x8B81
	#define GL_LINK_STATUS                    0x8B82
#endif

#ifndef GL_INFO_LOG_LENGTH
	#define		GL_INFO_LOG_LENGTH			0x8B84
#endif

#ifndef	GL_SAMPLE_BUFFERS_ARB
	#define	GL_SAMPLE_BUFFERS_ARB			0x80A8
	#define	GL_MULTISAMPLE_ARB				0x809D
#endif


Controller* sController = NULL;

const char* kVertexShader = 
				"void main()\n"
				"{\n"
				"	gl_Position = ftransform();\n"
				"	gl_FrontColor = gl_Color;\n"
				"	gl_BackColor = gl_Color;\n"
				"}\n";

const char* kFragmentShader =
				"void main()\n"
				"{\n"
				"	gl_FragColor = gl_Color;\n"
				"}\n";


Controller::Controller( HWND inMasterWindow,
				int inPixelFormat )
	: mMasterWindow( inMasterWindow )
	, mBlueprintWindow( CreateWindow( "Child", "", WS_CHILD|WS_VISIBLE, 0, 0, 198, 198, mMasterWindow, NULL, NULL, NULL ) )
	, mCameraWindow( CreateWindow( "Child", "", WS_CHILD|WS_VISIBLE, 0, 220, 966, 522, mMasterWindow, NULL, NULL, NULL ) )
	, mSmallCameraWindow( CreateWindow( "Child", "", WS_CHILD|WS_VISIBLE, 240, 0, 320, 173, mMasterWindow, NULL, NULL, NULL ) )
	, mPixelFormat( inPixelFormat )
	, mBlueprintGLContext( mBlueprintWindow, inPixelFormat )
	, mCameraGLContext( NULL )
	, mIsSwapNeeded( false )
{
	sController = this;

	mBlueprintGLContext.Initialize( 198, 198 );
	RenderBlueprintFrame();

	mTimer = ::SetTimer( mMasterWindow, 1, 50, Controller::TimerProc );
	LogMsg("Created timer %d", (int)mTimer );
}

void CALLBACK Controller::TimerProc( HWND window, UINT msg, UINT_PTR idevent, DWORD dTime )
{
	sController->Timed();
}

void	Controller::Timed()
{
	static int sWhichTime = 0;
	++sWhichTime;

	mIsSwapNeeded = (sWhichTime % 20) == 0;
	//LogMsg("Controller timer %d", sWhichTime);

	if (mIsSwapNeeded)
	{
		mIsSwapNeeded = false;
		CleanUpCameraContext();
		CleanUpSmallCameraContext();
	}
	else if (mCameraGLContext == NULL)
	{
		mCameraGLContext = new WinGLContext( mCameraWindow, mPixelFormat );
		::wglShareLists( mBlueprintGLContext.GLContext(), mCameraGLContext->GLContext() );
		mCameraGLContext->Initialize( 966, 522 );

		// First render to new context
		GetGLFuncs();
		SetUpVertexShader();
		SetUpFragmentShaderAndProgram();
		EachFrameSetup();
		EachFrameRender();
		mCameraGLContext->SwapBuffers();

		mSmallCameraGLContext = new WinGLContext( mSmallCameraWindow, mPixelFormat );
		::wglShareLists( mBlueprintGLContext.GLContext(), mSmallCameraGLContext->GLContext() );
		mSmallCameraGLContext->Initialize( 320, 173 );
		glClear( GL_COLOR_BUFFER_BIT );
		mSmallCameraGLContext->SwapBuffers();
	}
	else // later renders to big context
	{
		mCameraGLContext->MakeCurrent();
		EachFrameSetup();
		EachFrameRender();
		mCameraGLContext->SwapBuffers();
	}
}


void	Controller::RenderBlueprintFrame()
{
	mBlueprintGLContext.MakeCurrent();

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glFlush();

	mBlueprintGLContext.SwapBuffers();
}

void	Controller::CleanUpSmallCameraContext()
{
	if (mSmallCameraGLContext != NULL)
	{
		mSmallCameraGLContext->MakeCurrent();
		delete mSmallCameraGLContext;
		mSmallCameraGLContext = NULL;

		::InvalidateRect( mSmallCameraWindow, NULL, true );
		::UpdateWindow( mSmallCameraWindow );
	}
}

void	Controller::CleanUpCameraContext()
{
	if (mCameraGLContext != NULL)
	{
		mCameraGLContext->MakeCurrent();

		glDeleteProgram( mProgram );
		glDeleteShader( mVertexShader );

		delete mCameraGLContext;

		mCameraGLContext = NULL;

		::InvalidateRect( mCameraWindow, NULL, true );
		::UpdateWindow( mCameraWindow );
	}
}


void	Controller::GetGLFuncs()
{
	glCreateShader = (glCreateShaderProc) wglGetProcAddress("glCreateShader");
	glShaderSource = (glShaderSourceProc) wglGetProcAddress("glShaderSource");
	glCompileShader = (glCompileShaderProc) wglGetProcAddress("glCompileShader");
	glGetShaderiv = (glGetShaderivProc) wglGetProcAddress("glGetShaderiv");
	glCreateProgram = (glCreateProgramProc) wglGetProcAddress("glCreateProgram");
	glAttachShader = (glAttachShaderProc) wglGetProcAddress("glAttachShader");
	glDetachShader = (glDetachShaderProc) wglGetProcAddress("glDetachShader");
	glLinkProgram = (glLinkProgramProc) wglGetProcAddress("glLinkProgram");
	glGetProgramiv = (glGetProgramivProc) wglGetProcAddress("glGetProgramiv");
	glUseProgram = (glUseProgramProc) wglGetProcAddress("glUseProgram");
	glDeleteShader = (glDeleteShaderProc) wglGetProcAddress("glDeleteShader");
	glDeleteProgram = (glDeleteProgramProc) wglGetProcAddress("glDeleteProgram");
	glGetProgramInfoLog = (glGetProgramInfoLogProc) wglGetProcAddress("glGetProgramInfoLog");
	glGetShaderInfoLog = (glGetShaderInfoLogProc) wglGetProcAddress("glGetShaderInfoLog");
	glGetShaderSource = (glGetShaderSourceProc) wglGetProcAddress("glGetShaderSource");
}



void	Controller::EachFrameSetup()
{
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_VERTEX_PROGRAM_TWO_SIDE);
	glEnable(GL_COLOR_MATERIAL);
	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_LIGHTING);

	glDisable(GL_MULTISAMPLE_ARB);

	glClearColor(0, 0, 0, 1);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}

void	Controller::SetUpVertexShader()
{
	LogMsg("checkpoint 5.0");
	mVertexShader = glCreateShader(GL_VERTEX_SHADER);
	LogMsg("checkpoint 5.1");
	glShaderSource( mVertexShader, 1, &kVertexShader, NULL );
	LogMsg("checkpoint 5.2");
	glCompileShader( mVertexShader );
	LogMsg("checkpoint 5.3");
	GLint	status;
	glGetShaderiv( mVertexShader, GL_COMPILE_STATUS, &status );
	LogMsg("checkpoint 5.4");
	if (status == GL_FALSE)
	{
		LogMsg("==== Vertex shader failed to compile! ===");
		::KillTimer( mMasterWindow, mTimer );
		abort();
	}
}


void	Controller::SetUpFragmentShaderAndProgram()
{
	int fragShader = glCreateShader(GL_FRAGMENT_SHADER);
	glShaderSource( fragShader, 1, &kFragmentShader, NULL );
	glCompileShader( fragShader );
	GLint	status;
	glGetShaderiv( fragShader, GL_COMPILE_STATUS, &status );
	if (status == GL_FALSE)
	{
		LogMsg("==== Fragment shader failed to compile! ===");
		abort();
	}
	else
	{
		mProgram = glCreateProgram();
		glAttachShader( mProgram, mVertexShader );
		glAttachShader( mProgram, fragShader );
		glLinkProgram( mProgram );
		glDetachShader( mProgram, mVertexShader );
		glDetachShader( mProgram, fragShader );

		GLint	linkStatus;
		glGetProgramiv( mProgram, GL_LINK_STATUS, &linkStatus );
		if (linkStatus == GL_FALSE)
		{
			LogMsg("==== Program failed to link! ===");
			abort();
		}
	}
}

void	Controller::EachFrameRender()
{
	glUseProgram(mProgram);

	glBegin(GL_POINTS);
	float color[] = {1,1,1};
	glColor3fv(color);
	float vert[] = {0, 0, 0};
	glVertex3fv(vert);
	glEnd();

	glUseProgram(0);
	glFlush();
}
