#include "WinGLContext.h"

#include <gl/gl.h>

#define QO_PROCPTR_TYPE __stdcall *

typedef GLuint (QO_PROCPTR_TYPE glCreateShaderProc )(GLenum type);
typedef void (QO_PROCPTR_TYPE glShaderSourceProc )(GLuint shader,
													GLsizei count,
													const char* *string,
													const GLint *length);
typedef void (QO_PROCPTR_TYPE glCompileShaderProc )(GLuint shader);
typedef GLuint (QO_PROCPTR_TYPE glCreateProgramProc )(void);
typedef void (QO_PROCPTR_TYPE glAttachShaderProc )(GLuint program,
													GLuint shader);
typedef void (QO_PROCPTR_TYPE glDetachShaderProc ) (GLuint program,
													GLuint shader);
typedef void (QO_PROCPTR_TYPE glLinkProgramProc )(GLuint program);
typedef void (QO_PROCPTR_TYPE glUseProgramProc )(GLuint program);
typedef void (QO_PROCPTR_TYPE glDeleteProgramProc )(GLuint program);
typedef void (QO_PROCPTR_TYPE glDeleteShaderProc )(GLuint shader);
typedef void (QO_PROCPTR_TYPE glUniform1iProc )(GLint location, GLint v0);
typedef void (QO_PROCPTR_TYPE glUniform1fProc )(GLint location, GLfloat v0);
typedef void (QO_PROCPTR_TYPE glUniform1fvProc )(GLint location, GLsizei count,
												const GLfloat* values);
typedef void (QO_PROCPTR_TYPE glGetShaderivProc )(GLuint shader,
													GLenum pname,
													GLint *params);
typedef void (QO_PROCPTR_TYPE glGetProgramivProc )(GLuint program,
													GLenum pname,
													GLint *params);
typedef GLint (QO_PROCPTR_TYPE glGetUniformLocationProc )(GLuint program,
													const char *name);
typedef void (QO_PROCPTR_TYPE glGetProgramInfoLogProc )(GLuint program,
													GLsizei maxlength,
													GLsizei* outLength,
													GLbyte * infoLog );
typedef void (QO_PROCPTR_TYPE glGetShaderInfoLogProc )(GLuint shader,
													GLsizei maxlength,
													GLsizei* outLength,
													GLbyte * infoLog );
typedef void (QO_PROCPTR_TYPE glGetShaderSourceProc)( GLuint shder,
													GLsizei bufSize,
													GLsizei* outLength,
													char* outSource );


class Controller
{
public:
			Controller( HWND inMasterWindow,
				int inPixelFormat );


private:
	static void CALLBACK TimerProc( HWND window, UINT msg, UINT_PTR idevent, DWORD dTime );

	void			Timed();

	void			RenderBlueprintFrame();

	void			CleanUpCameraContext();
	void			CleanUpSmallCameraContext();
	void			GetGLFuncs();
	void			SetUpVertexShader();
	void			SetUpFragmentShaderAndProgram();
	void			EachFrameSetup();
	void			EachFrameRender();

	HWND			mMasterWindow;
	HWND			mBlueprintWindow;
	HWND			mCameraWindow;
	HWND			mSmallCameraWindow;
	int				mPixelFormat;
	WinGLContext	mBlueprintGLContext;
	WinGLContext*	mCameraGLContext;
	WinGLContext*	mSmallCameraGLContext;
	int				mVertexShader;
	int				mProgram;
	bool			mIsSwapNeeded;
	UINT_PTR		mTimer;

	glCreateShaderProc			glCreateShader;
	glShaderSourceProc			glShaderSource;
	glCompileShaderProc			glCompileShader;
	glGetShaderivProc			glGetShaderiv;
	glCreateProgramProc			glCreateProgram;
	glAttachShaderProc			glAttachShader;
	glDetachShaderProc			glDetachShader;
	glLinkProgramProc			glLinkProgram;
	glGetProgramivProc			glGetProgramiv;
	glUseProgramProc			glUseProgram;
	glDeleteShaderProc			glDeleteShader;
	glDeleteProgramProc			glDeleteProgram;
	glGetProgramInfoLogProc		glGetProgramInfoLog;
	glGetShaderInfoLogProc		glGetShaderInfoLog;
	glGetShaderSourceProc		glGetShaderSource;
};
