#pragma once

#if WIN32
	typedef	int									FFPixelFormat_Win;
#else
	#if QUESA_OS_COCOA
		typedef struct NSOpenGLPixelFormat_ob*	FFPixelFormat_Cocoa;
	#endif
	typedef	struct __AGLPixelFormatRec*			FFPixelFormat_Carbon;
#endif

enum ESBContextFormatOptions
{
	kSBContextFormat_Window				= 1L << 0,
	kSBContextFormat_PreferAccelerated	= 1L << 1,
	kSBContextFormat_PreferStencil		= 1L << 2,
	kSBContextFormat_PreferMultisample	= 1L << 3,
	kSBContextFormat_Stereo				= 1L << 4,
	kSBContextFormat_SingleBuffer		= 1L << 5
};

typedef unsigned int SBPixelFormatOptions;


/*!
	@function	FFChoosePixelFormat
	
	@abstract	Look for the best pixel format meeting some requirements.
	
	@discussion	We only consider formats that support OpenGL and RGB pixels,
				with a depth buffer at least 16 bits deep.
				
				We require a format that supports windowed rendering if the
				kSBContextFormat_Window flag is set, and require a format that
				supports rendering to a bitmap if that flag is clear.
				
				When kSBContextFormat_Window is set, we also require double
				buffering.
				
				The preferences are in this order of priority:
				
				* Accelerated if kSBContextFormat_PreferAccelerated is set,
				  software rendered if kSBContextFormat_PreferAccelerated is
				  clear.
				
				* 8 bits or more of stencil if kSBContextFormat_PreferStencil
				  is set.
				
				* Stereo if kSBContextFormat_Stereo is set.
				
				* Multisample capability if kSBContextFormat_PreferMultisample
				  is set, preferring 2 samples.
				
				* The more depth bits, the better.
				
				* The more color bits, the better.
	
	@param		inOptions		Some of the ESBContextFormatOptions values ored
								together.
	
	@result		A pixel format, or 0 on failure.
*/
#if WIN32
FFPixelFormat_Win	FFChoosePixelFormat_Win( SBPixelFormatOptions inOptions );
#else
FFPixelFormat_Carbon	FFChoosePixelFormat_Carbon( SBPixelFormatOptions inOptions );

	#if QUESA_OS_COCOA
	FFPixelFormat_Cocoa	FFChoosePixelFormat_Cocoa( SBPixelFormatOptions inOptions );
	#endif
#endif


/*!
	@function	FFDisposePixelFormat
	
	@abstract	Release a pixel format that is no longer needed.
	
	@param		inFormat		A pixel format.
*/
#if WIN32
void			FFDisposePixelFormat_Win( FFPixelFormat_Win inFormat );
#else
void			FFDisposePixelFormat_Mac( FFPixelFormat_Carbon inFormat );

	#if QUESA_OS_COCOA
	void		FFDisposePixelFormat_Mac( FFPixelFormat_Cocoa inFormat );
	#endif
#endif
