/*
 *  StDummyWindowGLContext.cpp
 *  SBEngine
 *
 *  Created by James Walker on 12/13/07.
 *  Copyright 2007 ScriptPerfection Enterprises, Inc.. All rights reserved.
 *
 */
#include "stdafx.h"
#include "StDummyWindowGLContext.h"

	#include <windows.h>
	#include <wingdi.h>

#include <cstring>


static const char*	kWindowClassName	= "SBE.dummy";

HDC		gDummyGLDC = NULL;

static void RegisterWindowClass( HINSTANCE inInstance )
{
	WNDCLASSEX wcex;

	wcex.cbSize        = sizeof(WNDCLASSEX); 
	wcex.style         = CS_OWNDC;
	wcex.lpfnWndProc   = (WNDPROC) DefWindowProc;
	wcex.cbClsExtra    = 0;
	wcex.cbWndExtra    = 0;
	wcex.hInstance     = inInstance;
	wcex.hIcon         = LoadIcon(NULL, IDI_WINLOGO);
	wcex.hCursor       = LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground = NULL;
	wcex.lpszMenuName  = NULL;
	wcex.lpszClassName = kWindowClassName;
	wcex.hIconSm       = NULL;

	RegisterClassEx(&wcex);
}

class XDummyWindowGLContext
{
public:
				XDummyWindowGLContext();
				~XDummyWindowGLContext();

	void		MakeCurrent();
	
private:
	HWND		mWindow;
	HDC			mDC;
	HGLRC		mGLContext;
};

XDummyWindowGLContext::XDummyWindowGLContext()
{
	HINSTANCE	theInstance = GetModuleHandle( NULL );
	DWORD	err;
	
	RegisterWindowClass( theInstance );
	
	mWindow = CreateWindow( kWindowClassName, "XDummy",
									WS_OVERLAPPEDWINDOW |
									WS_CLIPCHILDREN     |
									WS_CLIPSIBLINGS,
									400, 30, 40, 40,
									NULL, NULL, theInstance, NULL );
	if (mWindow == NULL)
	{
		err = GetLastError();
		throw err;
	}

	mDC = GetDC( mWindow );
	
	PIXELFORMATDESCRIPTOR	pixelFormatDesc;

	std::memset( &pixelFormatDesc, 0, sizeof(pixelFormatDesc) );
		
	pixelFormatDesc.nSize      = sizeof(pixelFormatDesc);
	pixelFormatDesc.nVersion   = 1;
	pixelFormatDesc.dwFlags    = PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL;
	pixelFormatDesc.iPixelType = PFD_TYPE_RGBA;
	    
	int pixelFormat = ChoosePixelFormat( mDC, &pixelFormatDesc );

	if (pixelFormat == 0)
	{
		err = GetLastError();
		ReleaseDC( mWindow, mDC );
		DestroyWindow( mWindow );
		throw err;
	}

	DescribePixelFormat( mDC, pixelFormat, sizeof(pixelFormatDesc),
		&pixelFormatDesc );
	
	if (not SetPixelFormat( mDC, pixelFormat, &pixelFormatDesc ))
	{
		err = GetLastError();
		ReleaseDC( mWindow, mDC );
		DestroyWindow( mWindow );
		throw err;
	}
	
	mGLContext = wglCreateContext( mDC );
	
	if (mGLContext == NULL)
	{
		err = GetLastError();
		ReleaseDC( mWindow, mDC );
		DestroyWindow( mWindow );
		throw err;
	}
	
	gDummyGLDC = mDC;
}

XDummyWindowGLContext::~XDummyWindowGLContext()
{
	wglMakeCurrent( NULL, NULL );
	wglDeleteContext( mGLContext );
	gDummyGLDC = NULL;
	ReleaseDC( mWindow, mDC );
	DestroyWindow( mWindow );
}

void	XDummyWindowGLContext::MakeCurrent()
{
	wglMakeCurrent( mDC, mGLContext );
}



StDummyWindowGLContext::StDummyWindowGLContext()
	: mImp( new XDummyWindowGLContext )
{
	MakeCurrent();
}

StDummyWindowGLContext::~StDummyWindowGLContext()
{
}

void	StDummyWindowGLContext::MakeCurrent()
{
	mImp->MakeCurrent();
}
