#include "stdafx.h"

#include "WinGLContext.h"

#include <gl/gl.h>

WinGLContext::WinGLContext(
					HWND inWindow,
					int inPixelFormat )
	: mWindow( inWindow )
	, mDC( ::GetDC( inWindow ) )
	, mGLContext( 0 )
{
	PIXELFORMATDESCRIPTOR	pixelFormatDesc;
	::DescribePixelFormat( mDC, inPixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pixelFormatDesc);

	if (not ::SetPixelFormat( mDC, inPixelFormat, &pixelFormatDesc ))
	{
		LogMsg("SetPixelFormat failed!!");
		abort();
	}

	mGLContext = ::wglCreateContext(mDC);

	LogMsg("Created a WinGLContext %p", mGLContext);
}

WinGLContext::~WinGLContext()
{
	LogMsg("Destroying a WinGLContext %p", mGLContext);
	::wglDeleteContext( mGLContext );
	::ReleaseDC( mWindow, mDC );
}

void	WinGLContext::Initialize(
					int inWidth,
					int inHeight )
{
	::wglMakeCurrent( mDC, mGLContext );

	GLint			viewPort[4];
	viewPort[0] = viewPort[1] = 0;
	viewPort[2] = inWidth;
	viewPort[3] = inHeight;
	glEnable( GL_SCISSOR_TEST );
	glScissor( viewPort[0], viewPort[1], viewPort[2], viewPort[3] );
	glViewport( viewPort[0], viewPort[1], viewPort[2], viewPort[3] );
}


void	WinGLContext::SwapBuffers()
{
	glFlush();
	::SwapBuffers( mDC );
}

void	WinGLContext::MakeCurrent()
{
	::wglMakeCurrent( mDC, mGLContext );
}

